@@ -17,13 +17,29 @@
 #define MAX(a, b) ((a) > (b) ? (a) : (b))
 #define MIN(a, b) ((a) < (b) ? (a) : (b))
 
-#if LIBAVCODEC_VERSION_INT < AV_VERSION_INT(54, 28, 0)
-#define avcodec_free_frame av_freep
+int64_t get_default_channel_layout(int nb_channels)
+{
+/* 51.8.0 for FFmpeg, 51.26.0 for libav. I don't know how to identify them,
+   so let's use the higher one. I really wish Ubuntu would stop being
+   stupid and just drop libav. */
+#if LIBAVUTIL_VERSION_INT >= AV_VERSION_INT(51, 26, 0)
+	return av_get_default_channel_layout(nb_channels);
+#else
+	switch (nb_channels) {
+		case 1:
+			return AV_CH_LAYOUT_MONO;
+		case 2:
+			return AV_CH_LAYOUT_STEREO;
+		default:
+			return 0;
+	}
 #endif
+}
 
 int decode_audio_file(ChromaprintContext *chromaprint_ctx, const char *file_name, int max_length, int *duration)
 {
-	int ok = 0, remaining, length, consumed, codec_ctx_opened = 0, got_frame, stream_index;
+	int ok = 0, length, consumed, codec_ctx_opened = 0, got_frame, stream_index, last_chunk = 0;
+	unsigned long long remaining;
 	AVFormatContext *format_ctx = NULL;
 	AVCodecContext *codec_ctx = NULL;
 	AVCodec *codec = NULL;
@@ -80,7 +96,7 @@ int decode_audio_file(ChromaprintContext *chromaprint_ctx, const char *file_name
 	if (codec_ctx->sample_fmt != AV_SAMPLE_FMT_S16) {
 		int64_t channel_layout = codec_ctx->channel_layout;
 		if (!channel_layout) {
-			channel_layout = av_get_default_channel_layout(codec_ctx->channels);
+			channel_layout = get_default_channel_layout(codec_ctx->channels);
 		}
 #if defined(HAVE_SWRESAMPLE)
 		convert_ctx = swr_alloc_set_opts(NULL,
@@ -131,7 +147,7 @@ int decode_audio_file(ChromaprintContext *chromaprint_ctx, const char *file_name
 	remaining = max_length * codec_ctx->channels * codec_ctx->sample_rate;
 	chromaprint_start(chromaprint_ctx, codec_ctx->sample_rate, codec_ctx->channels);
 
-	frame = avcodec_alloc_frame();
+	frame = av_frame_alloc();
 
 	while (1) {
 		if (av_read_frame(format_ctx, &packet) < 0) {
@@ -139,7 +155,7 @@ int decode_audio_file(ChromaprintContext *chromaprint_ctx, const char *file_name
 		}
 
 		if (packet.stream_index == stream_index) {
-			avcodec_get_frame_defaults(frame);
+			av_frame_unref(frame);
 
 			got_frame = 0;
 			consumed = avcodec_decode_audio4(codec_ctx, frame, &got_frame, &packet);
@@ -170,16 +186,21 @@ int decode_audio_file(ChromaprintContext *chromaprint_ctx, const char *file_name
 					}
 					data = dst_data;
 				}
-				length = MIN(remaining, frame->nb_samples * codec_ctx->channels);
+
+				length = frame->nb_samples * codec_ctx->channels;
+				if (max_length > 0) {
+					if (remaining < length) {
+						length = remaining;
+						last_chunk = 1;
+					}
+				}
+
 				if (!chromaprint_feed(chromaprint_ctx, data[0], length)) {
 					goto done;
 				}
 
-				if (max_length) {
-					remaining -= length;
-					if (remaining <= 0) {
-						goto finish;
-					}
+				if (last_chunk) {
+					goto finish;
 				}
 			}
 		}
@@ -196,7 +217,7 @@ finish:
 
 done:
 	if (frame) {
-		avcodec_free_frame(&frame);
+		av_frame_free(&frame);
 	}
 	if (dst_data[0]) {
 		av_freep(&dst_data[0]);
@@ -223,7 +244,7 @@ int fpcalc_main(int argc, char **argv)
 	int32_t *raw_fingerprint;
 	char *file_name, *fingerprint, **file_names;
 	ChromaprintContext *chromaprint_ctx;
-	int algo = CHROMAPRINT_ALGORITHM_DEFAULT, num_failed = 0;
+	int algo = CHROMAPRINT_ALGORITHM_DEFAULT, num_failed = 0, do_hash = 0;
 
 	file_names = malloc(argc * sizeof(char *));
 	for (i = 1; i < argc; i++) {
@@ -238,6 +259,9 @@ int fpcalc_main(int argc, char **argv)
 		else if (!strcmp(arg, "-raw")) {
 			raw = 1;
 		}
+		else if (!strcmp(arg, "-hash")) {
+			do_hash = 1;
+		}
 		else if (!strcmp(arg, "-algo") && i + 1 < argc) {
 			const char *v = argv[++i];
 			if (!strcmp(v, "test1")) { algo = CHROMAPRINT_ALGORITHM_TEST1; }
@@ -263,6 +287,7 @@ int fpcalc_main(int argc, char **argv)
 		printf("  -length SECS  length of the audio data used for fingerprint calculation (default 120)\n");
 		printf("  -raw          output the raw uncompressed fingerprint\n");
 		printf("  -algo NAME    version of the fingerprint algorithm\n");
+		printf("  -hash         calculate also the fingerprint hash\n");
 		return 2;
 	}
 
@@ -317,6 +342,11 @@ int fpcalc_main(int argc, char **argv)
 			printf("FINGERPRINT=%s\n", fingerprint);
 			chromaprint_dealloc(fingerprint);
 		}
+        if (do_hash) {
+            int32_t hash = 0;
+            chromaprint_get_fingerprint_hash(chromaprint_ctx, &hash);
+            printf("HASH=%d\n", hash);
+        }
 	}
 
 	chromaprint_free(chromaprint_ctx);
