@@ -46,7 +46,7 @@ extern "C" {
 typedef void *ChromaprintContext;
 
 #define CHROMAPRINT_VERSION_MAJOR 1
-#define CHROMAPRINT_VERSION_MINOR 1
+#define CHROMAPRINT_VERSION_MINOR 2
 #define CHROMAPRINT_VERSION_PATCH 0
 
 enum ChromaprintAlgorithm {
@@ -187,6 +187,20 @@ CHROMAPRINT_API int chromaprint_get_fingerprint(ChromaprintContext *ctx, char **
  */
 CHROMAPRINT_API int chromaprint_get_raw_fingerprint(ChromaprintContext *ctx, void **fingerprint, int *size);
 
+/**
+ * Return 32-bit hash of the calculated fingerprint.
+ *
+ * See chromaprint_hash_fingerprint() for details on how to use the hash.
+ *
+ * Parameters:
+ *  - ctx: Chromaprint context pointer
+ *  - hash: pointer to a 32-bit integer where the hash will be stored
+ *
+ * Returns:
+ *  - 0 on error, 1 on success
+ */
+CHROMAPRINT_API int chromaprint_get_fingerprint_hash(ChromaprintContext *ctx, void *hash);
+
 /**
  * Compress and optionally base64-encode a raw fingerprint
  *
@@ -210,7 +224,7 @@ CHROMAPRINT_API int chromaprint_get_raw_fingerprint(ChromaprintContext *ctx, voi
  * Returns:
  *  - 0 on error, 1 on success
  */
-CHROMAPRINT_API int chromaprint_encode_fingerprint(void *fp, int size, int algorithm, void **encoded_fp, int *encoded_size, int base64);
+CHROMAPRINT_API int chromaprint_encode_fingerprint(const void *fp, int size, int algorithm, void **encoded_fp, int *encoded_size, int base64);
 
 /**
  * Uncompress and optionally base64-decode an encoded fingerprint
@@ -233,7 +247,31 @@ CHROMAPRINT_API int chromaprint_encode_fingerprint(void *fp, int size, int algor
  * Returns:
  *  - 0 on error, 1 on success
  */
-CHROMAPRINT_API int chromaprint_decode_fingerprint(void *encoded_fp, int encoded_size, void **fp, int *size, int *algorithm, int base64);
+CHROMAPRINT_API int chromaprint_decode_fingerprint(const void *encoded_fp, int encoded_size, void **fp, int *size, int *algorithm, int base64);
+
+/**
+ * Generate a single 32-bit hash for a raw fingerprint.
+ *
+ * If two fingerprints are similar, their hashes generated by this function
+ * will also be similar. If they are significantly different, their hashes
+ * will most likely be significantly different as well, but you can't rely
+ * on that.
+ *
+ * You compare two hashes by counting the bits in which they differ. Normally
+ * that would be something like POPCNT(hash1 XOR hash2), which returns a
+ * number between 0 and 32. Anthing above 15 means the hashes are
+ * completely different.
+ *
+ * Parameters:
+ *  - fp: pointer to an array of 32-bit integers representing the raw
+ *        fingerprint to be hashed
+ *  - size: number of items in the raw fingerprint
+ *  - hash: pointer to a 32-bit integer where the hash will be stored
+ *
+ * Returns:
+ *  - 0 on error, 1 on success
+ */
+CHROMAPRINT_API int chromaprint_hash_fingerprint(const void *fp, int size, void *hash);
 
 /**
  * Free memory allocated by any function from the Chromaprint API.
